@@ -1,13 +1,9 @@
 from .crawler import Crawler
 from .http import CachedWeb, FetchError, Web
-from .link import EggLink, Link, SourceLink
 
 __all__ = (
   CachedWeb,
   Crawler,
-  EggLink,
   FetchError,
-  Link,
-  SourceLink,
   Web,
 )
